const $isset = (flag: number, sequence = 0) => (sequence & flag) === flag
const $set = (flag: number, sequence = 0) => sequence | flag
const $unset = (flag: number, sequence = 0) => sequence & ~flag
const $toggle = (flag: number, sequence = 0) => sequence ^ flag

const combine = (flags: number[]) => flags.reduce((flags, flag) => flags + flag, 0)

type FlagHandler<T> = (flag: number, flags: number) => T
const createMultiFlagHandler = <T extends number | boolean>(fn: FlagHandler<T>) => 
    (flag: number | number[], sequence = 0): T => 
        fn(Array.isArray(flag) ? combine(flag) : flag, sequence) 

const set = createMultiFlagHandler($set)
const unset = createMultiFlagHandler($unset)
const toggle = createMultiFlagHandler($toggle)
const isset = createMultiFlagHandler($isset)

export {
    isset,
    set,
    unset,
    toggle
}